@@ -1,14 +1,14 @@
 /*
  * makemat.c - make table file and matrix file
  *
- * $Id: makemat.c,v 1.12 2003/06/04 14:19:35 kazuma-t Exp $
+ * $Id: makemat.c,v 1.2 2007/03/13 07:51:47 masayu-a Exp $
  */
 
 #include "chadic.h"
 #include "literal.h"
 
 #define CTYPE_MAX        500
-#define REN_TBL_MAX      5000
+#define REN_TBL_MAX      10000
 #define RENSETU_CELL_MAX (8192*4)
 
 #define HINSI_ID_MAX USHRT_MAX
@@ -39,8 +39,7 @@ static rensetu_pair_t rensetu_tbl[RENSETU_CELL_MAX];
 static int i_num;
 static int j_num;
 
-static connect_rule_t **connect_mtr1;  /* original connection matrix */
-static connect_rule_t **connect_mtr2;  /* compressed connection matrix */
+static connect_rule_t **connect_mtr;
 typedef unsigned char rensetu_mtr_t;
 
 /*
@@ -367,7 +366,7 @@ add_connect_rule(int in, int prev, int cost, int is_last, int *in_cells,
 	/*
 	 * ǸʻǤʤϵ§񤭤ʤ 
 	 */
-	if (!is_last && connect_mtr1[cur][in].cost)
+	if (!is_last && connect_mtr[cur][in].cost)
 	    continue;
 	suffix_len_max = 0;
 	/*
@@ -395,8 +394,8 @@ add_connect_rule(int in, int prev, int cost, int is_last, int *in_cells,
 	 * §ɲ 
 	 */
 	if (suffix_len_max) {
-	    connect_mtr1[cur][in].next = next - in;
-	    connect_mtr1[cur][in].cost = cost < 0 ? 0 : cost + 1;
+	    connect_mtr[cur][in].next = next - in;
+	    connect_mtr[cur][in].cost = cost < 0 ? 0 : cost + 1;
 	}
     }
 }
@@ -514,20 +513,11 @@ read_rensetu(FILE * fp, int lines)
 	(connect_rule_t *) cha_malloc(sizeof(connect_rule_t) * cell_num *
 				      tbl_num);
     memset(ptr, 0, sizeof(connect_rule_t) * cell_num * tbl_num);
-    connect_mtr1 =
+    connect_mtr =
 	(connect_rule_t **) cha_malloc(sizeof(connect_rule_t *) *
 				       cell_num);
     for (c1 = 0; c1 < cell_num; c1++)
-	connect_mtr1[c1] = ptr + c1 * tbl_num;
-
-    ptr =
-	(connect_rule_t *) cha_malloc(sizeof(connect_rule_t) * cell_num *
-				      tbl_num);
-    connect_mtr2 =
-	(connect_rule_t **) cha_malloc(sizeof(connect_rule_t *) *
-				       cell_num);
-    for (c1 = 0; c1 < cell_num; c1++)
-	connect_mtr2[c1] = ptr + c1 * tbl_num;
+	connect_mtr[c1] = ptr + c1 * tbl_num;
 
     in_cells = cha_malloc(sizeof(int) * cell_num);
     cur_cells = cha_malloc(sizeof(int) * cell_num);
@@ -539,8 +529,14 @@ read_rensetu(FILE * fp, int lines)
      * û§ν˽ 
      */
     for (rlen = 2; rlen <= rule_len_max; rlen++) {
-	fprintf(stderr, rlen == 2 ? "bi%s" : rlen == 3 ? "tri%s" : "%d%s",
-		"-gram: ", rlen);
+		/*	fprintf(stderr, rlen == 2 ? "bi%s" : rlen == 3 ? "tri%s" : "%d%s",
+			"-gram: ", rlen); */
+		if (rlen <= 3)	
+			fprintf(stderr, rlen == 2 ? "bi%s" : rlen == 3 ? "tri%s" : "%s",
+					"-gram: ");	
+		else	
+			fprintf(stderr, "%d-gram: ", rlen);
+
 	for (ln = 0; ln < linenum; ln++) {
 	    if (rule_len[ln] != rlen)
 		continue;
@@ -619,8 +615,8 @@ compare_vector1(int k, int j, int num)
     int i;
 
     for (i = 0; i < num; i++)
-	if (connect_mtr2[i][k].next != connect_mtr1[i][j].next ||
-	    connect_mtr2[i][k].cost != connect_mtr1[i][j].cost)
+	if (connect_mtr[i][k].next != connect_mtr[i][j].next ||
+	    connect_mtr[i][k].cost != connect_mtr[i][j].cost)
 	    return 0;
 
     return 1;
@@ -632,8 +628,8 @@ copy_vector1(int j, int j_n, int num)
     int i;
 
     for (i = 0; i < num; i++) {
-	connect_mtr2[i][j_n].next = connect_mtr1[i][j].next;
-	connect_mtr2[i][j_n].cost = connect_mtr1[i][j].cost;
+	connect_mtr[i][j_n].next = connect_mtr[i][j].next;
+	connect_mtr[i][j_n].cost = connect_mtr[i][j].cost;
     }
 }
 
@@ -643,8 +639,8 @@ compare_vector2(int k, int i, int num)
     int j;
 
     for (j = 0; j < num; j++)
-	if (connect_mtr2[i][j].next != connect_mtr1[k][j].next ||
-	    connect_mtr2[i][j].cost != connect_mtr1[k][j].cost)
+	if (connect_mtr[i][j].next != connect_mtr[k][j].next ||
+	    connect_mtr[i][j].cost != connect_mtr[k][j].cost)
 	    return 0;
 
     return 1;
@@ -656,8 +652,8 @@ copy_vector2(int i, int i_n, int num)
     int j;
 
     for (j = 0; j < num; j++) {
-	connect_mtr1[i_n][j].next = connect_mtr2[i][j].next;
-	connect_mtr1[i_n][j].cost = connect_mtr2[i][j].cost;
+	connect_mtr[i_n][j].next = connect_mtr[i][j].next;
+	connect_mtr[i_n][j].cost = connect_mtr[i][j].cost;
     }
 }
 
@@ -673,39 +669,41 @@ condense_matrix(void)
 
     fprintf(stderr, "matrix size: %dx%d", cell_num, tbl_num);
 
-    for (i = 0; i < cell_num; i++) {
-	rensetu_tbl[i].i_pos = i;
-	rensetu_tbl[i].j_pos = i;
-    }
-
     for (j = 0; j < tbl_num; j++) {
+	int has_same = 0;
+
 	for (k = 0; k < j_n; k++) {
 	    if (compare_vector1(k, j, cell_num)) {
 		rensetu_tbl[j].j_pos = k;
+		has_same = 1;
 		break;
 	    }
 	}
-	if (rensetu_tbl[j].j_pos == j) {
-	    copy_vector1(j, j_n, cell_num);
+	if (!has_same) {
+	    if (j != j_n)
+		copy_vector1(j, j_n, cell_num);
 	    rensetu_tbl[j].j_pos = j_n++;
 	}
     }
+    j_num = j_n;
 
     for (i = 0; i < cell_num; i++) {
+	int has_same = 0;
+
 	for (k = 0; k < i_n; k++) {
-	    if (compare_vector2(k, i, j_n)) {
+	    if (compare_vector2(k, i, j_num)) {
 		rensetu_tbl[i].i_pos = k;
+		has_same = 1;
 		break;
 	    }
 	}
-	if (rensetu_tbl[i].i_pos == i) {
-	    copy_vector2(i, i_n, j_n);
+	if (!has_same) {
+	    if (i != i_n)
+		copy_vector2(i, i_n, j_num);
 	    rensetu_tbl[i].i_pos = i_n++;
 	}
     }
-
     i_num = i_n;
-    j_num = j_n;
 
     /*
      * print for check 
@@ -759,11 +757,11 @@ write_matrix(void)
 
     for (i = 0; i < i_num; i++) {
 	int nval = 0;
-	int next0 = connect_mtr1[i][0].next;
-	int cost0 = connect_mtr1[i][0].cost;
+	int next0 = connect_mtr[i][0].next;
+	int cost0 = connect_mtr[i][0].cost;
 	for (j = 0; j < j_num; j++) {
-	    if (connect_mtr1[i][j].next == next0 &&
-		connect_mtr1[i][j].cost == cost0) {
+	    if (connect_mtr[i][j].next == next0 &&
+		connect_mtr[i][j].cost == cost0) {
 		nval++;
 	    } else {
 		if (next0 == 0 && cost0 == 0)
@@ -773,8 +771,8 @@ write_matrix(void)
 		else
 		    fprintf(fp, "%d,%dx%d ", next0, cost0, nval);
 		nval = 1;
-		next0 = connect_mtr1[i][j].next;
-		cost0 = connect_mtr1[i][j].cost;
+		next0 = connect_mtr[i][j].next;
+		cost0 = connect_mtr[i][j].cost;
 	    }
 	}
 	if (nval > 0) {
@@ -890,9 +888,6 @@ in rensetu_pair_t       v
   ----
 
   * matrix.cha
-  compressed connection matrix -- `connect_mtr2' in mkchadic/makemat.c
-  connect_mtr1 is compressed to connect_mtr2.
-
   -----
   0,394 0,8001 0,3430 0,8001 0,3766 0,8001x2 0,3094 ....
   -----
